<?php
/**
* WP e-Commerce Canada Post shipping module
* @author : Yann Bouschet <contact@yannbouyschet.com> 
*      URL: http://www.yannbouschet.com
* @version : 0.1 
* @date   11/05/2010
* Purpose : Link Canada Post shipping API to WP e-commerce
* Released : Under GPL 
*/
 
require_once "class.xmltoarray.php"; 
 
class canadapost {
	var $internal_name, $name, $merchantCPCID;
	
	/**
	 * Shipping module settings
	 *
	 * @var Array
	 */
	var $settings;
	
	var $base_country;
	var $base_zipcode;
	
	/**
	 * Constructor
	 */
	function canadapost () {
		$this->internal_name = "canadapost";
		$this->name = __('Canada Post', 'wpsc');
		$this->is_external = true;
		$this->requires_weight = true;
		$this->needs_zipcode = true;
		
		$shipping = get_option('canadapost_options');
		$this->merchantCPCID = $shipping['charge'];
						
		$this->base_country = get_option('base_country');
		$this->base_zipcode = get_option('base_zipcode');
				
		return true;
	}
	
	function getId() {
	}
	
	function setId($id) {
	}
	
	function getName() {
		return $this->name;
	}
	
	function getInternalName() {
		return $this->internal_name;
	}
	
	function getMerchantCPCID() {
		return $this->merchantCPCID;
	}

	function getForm() {
		// Only for Canadan merchants
		if ($this->base_country != 'CA') {
			return __('This shipping module only works if the base country in settings, region is set to Canada.', 'wpsc');
		}
		
		// Base postcode must be set
		if (strlen($this->base_zipcode) <6) {
			return __('You must set your base postcode above before this shipping module will work.', 'wpsc');
		}
		
		$shipping = get_option('canadapost_options');
		
		$output .= '<tr>';
		$output .= '	<td>';
		$output .= '		MerchantCPCID:<br/>';
		$output .= '		<input type="text" name="shipping[charge]" value="'.htmlentities($shipping['charge']).'"><br/>';
		$output .= '	</td>';
		$output .= '</tr>';

		return $output;
	}
	
	function submit_form() {				
		if($_POST['shipping'] != null) {

			$shipping = (array)get_option('canadapost_options');
			$submitted_shipping = (array)$_POST['shipping'];

			update_option('canadapost_options',array_merge($shipping, $submitted_shipping));

		}
		
		return true;
	}
	
	function get_item_shipping() {
	}
	
	function convertSize($size,$unit) {
		switch ($unit) {
			// we need the units in cm
			case 'cm':
				// change nothing
				$finalSize = $size;
				break;
			case 'meter':
				// convert from m to cm
				$finalSize = $size * 100;
				break;
			case 'in':
				// convert from in to cm
				$finalSize = $size * 2.54;
				break;
			}
		return $finalSize;
	}
					
	function getQuote() {
		global $wpdb, $wpsc_cart;
				
		if ($this->base_country != 'CA' || strlen($this->base_zipcode) <6 || !count($wpsc_cart->cart_items)) return;
		
		if (isset($_POST['country'])) {

			$country = $_POST['country'];
			$_SESSION['wpsc_delivery_country'] = $country;

		} else {

			$country = $_SESSION['wpsc_delivery_country'];

		}
		
		$city = $_SESSION['wpsc_delivery_city'];
		$state = $_SESSION['wpsc_delivery_state'];
		
		//WP e-commerce always ouput Weight in pounds but we need kilograms for Canado Post
		$weight = floatval(wpsc_cart_weight_total() * 0.45359237);

		$destzipcode = '';
		if(isset($_POST['zipcode'])) {
			$destzipcode = $_POST['zipcode'];      
			$_SESSION['wpsc_zipcode'] = $_POST['zipcode'];
		} else if(isset($_SESSION['wpsc_zipcode'])) {
			$destzipcode = $_SESSION['wpsc_zipcode'];
		}
		
		//formation du XML d'envoi
		$url = "http://cybervente.postescanada.ca:30000";
		
		$xmlrequest = "<?xml version=\"1.0\" ?>";
/*/*
***********************************************************************
Ce fichier est un exemple de document XML envoye au serveur CyberVente 
en vue d'obtenir les frais d'expedition et les dates de livraison des 
articles deposes dans un panier d'achat.

Pour envoyer la requete, ouvrir une connexion TCP/IP &agrave; :
     Adresse IP : cybervente.postescanada.ca 
     Port TCP : 30000

Vous pouvez utiliser la methode HTTP POST pour transmettre ce document XML 
au serveur (meme adresse IP et meme port TCP). Nous vous suggerons fortement 
d'aller au URL suivant pour voir un exemple interactif d'une requete 
envoyee par HTTP POST : 
	http://cybervente.postescanada.ca/DevelopersResources/protocolV3/HTTPInterface.html

Nota : L'adresse ci-dessus est utilisee a des fins d'essai. Si votre site Web 
       est pret a lancer, veuillez communiquer avec cybervente@postescanada.ca 
       pour demander l'adresse IP de notre serveur de production (ou consultez 
       la FAQ).
***********************************************************************
*/
		$xmlrequest .="<!DOCTYPE eparcel SYSTEM \"eParcel.dtd\" >\n";
		$xmlrequest .="<eparcel>\n";
/*
   ************************************ 
   * Choix de langue :
   *      en=ANGLAIS
   *      fr=FRANCAIS
   * Nota : Ce parametre est FACULTATIF (valeur par defaut = anglais).
   ************************************ */
		$xmlrequest .="<language>fr</language>\n"; 

		$xmlrequest .="<ratesAndServicesRequest>\n";
/*     
      ************************************ 
      * Code de detaillant attribue par Postes Canada 
      * Si vous n'en avez pas, adressez-vous a 
      * cybervente@postescanada.ca pour en demander un.
      * Nota : Le serveur CyberVente utilise le code 
      * de detaillant pour extraire :
      *      - la liste des boites d'emballage;
      *      - le code postal d'origine;
      *      - ...
      ************************************ */
      $xmlrequest .="<merchantCPCID> ".$this->merchantCPCID." </merchantCPCID>\n";

		/*
      **********************************
      Code postal d'origine
      Ce parametre remplacera le code postal 
      qui figure dans le profil du detaillant.
      Nota : Ce parametre est FACULTATIF.
      ***********************************/               
		$xmlrequest .="<fromPostalCode>".$this->base_zipcode."</fromPostalCode>\n";   

      /*
      **********************************
      Delai d'execution de la commande (en heures)
      Si vous l'entrez ici, ce parametre 
      remplacera le delai indique dans le 
      profil du detaillant.
      Nota : Ce param&etre est FACULTATIF.
      ***********************************/               
	//<turnAroundTime> </turnAroundTime>      

      /*
      ************************************ 
      Prix total des articles enumeres dans la requete.
      Le prix est exprime en dollars canadiens et ne
      doit pas inclure les taxes.
      Le prix des articles sert a calculer le cout
      de l'assurance et de l'option Signature.
      Nota : Ce parametre est FACULTATIF.
      ************************************ */
      //<itemsPrice> {Ins&#233;rer le prix des articles} </itemsPrice>

      /*
      ************************************ 
      * Inserer ici les articles deposes dans le panier d'achat.
      * L'exemple ci-dessous montre deux articles differents.
      * Chaque article est defini au moyen des attributs suivants :
      *         - Quantite
      *         - Poids (en kg) 
      *         - Longueur, largeur et hauteur (en cm)
      *         - Description
      ************************************ 
      */
      $xmlrequest .="<lineItems>\n";
      
      //boucle
        
      foreach((array)$wpsc_cart->cart_items as $cart_item) {
      		$xmlrequest .="<item>\n";
      		 
      		$quantity = $cart_item->quantity;
			$weight = $cart_item->weight;
      		
      		$xmlrequest .="<quantity>".$quantity."</quantity>\n";
            $xmlrequest .="<weight>".$weight."</weight>\n";
            
      		//rcupre et converti les dimensions
			$meta = get_product_meta($cart_item->product_id,'dimensions');
			
			if ($meta && is_array($meta)) {
				$tempoLength = $meta["length"];
				$tempoWidth = $meta["width"];
				$tempoHeight = $meta["height"];
				
				$length_unit = $meta["length_unit"];
				$width_unit = $meta["width_unit"];
				$height_unit = $meta["height_unit"];
				
				//Canada Post  besoin de centimtres et de kg pour fonctionner
				
				$length = $this->convertSize($tempoLength, $length_unit);
				$width = $this->convertSize($tempoWidth, $width_unit);
				$height = $this->convertSize($tempoHeight, $height_unit);
			}
			
			$xmlrequest .="<length>".$length."</length>\n";
            $xmlrequest .="<width>".$width."</width>\n";
            $xmlrequest .="<height>".$height."</height>\n";
			
			$description = $cart_item->name;
			
			$xmlrequest .="<description> book ".$description."</description>\n";
			
			$xmlrequest .="</item>\n";
		}
				
      
	 //fin de boucle  
       
      $xmlrequest .="</lineItems>\n";

      /*
      ************************************ 
      Adresse de destination

      (*) Pour une destination canadienne, les champs 'pays' et 'code postal' 
      doivent contenir des donnees valides (les entrees 'ville' et 'province ou 
      etat' ne sont pas utilisees).

      (*) Pour une destination aux Etats-Unis, les champs 'pays' et 'province ou 
      etat' doivent contenir des donnees valides (les entrees 'ville' et 'code 
      postal' ne sont pas utilisees pour l'instant mais le code ZIP sera utilise 
      eventuellement). Vous pouvez entrer le nom de l'etat au long ou le code 
      d'etat.

      (*) Pour une destination internationale, seul le champ 'pays' doit 
      contenir une donnee valide (les entrees 'ville', 'code postal' et 
      'province ou etat' ne sont pas utilisees). Vous pouvez entrer le nom du 
      pays au long ou le code de pays a 2 lettres (norme ISO 3166).
      ************************************ 
      */
     
		$xmlrequest .="<city> ".$city."</city>\n";
      	$xmlrequest .="<provOrState> ".$state."</provOrState>\n";
		$xmlrequest .="<country> ".$country."</country>\n";
		$xmlrequest .="<postalCode>".$destzipcode."</postalCode>\n";

		$xmlrequest .="</ratesAndServicesRequest>\n";

		$xmlrequest .="</eparcel>\n";

		$response = wp_remote_post($url,array('body'=>array('request'=>serialize($xmlrequest))));

		
		// test $result['response'] and if OK do something with $result['body']

		/
		if(is_wp_error($response))
		{
			$error_string = $response->get_error_message();
			echo '<div id="message" class="error"><p>' . $error_string . '</p></div>';
		} else {
			$xml_data =  $response['body'];
			
			$xmlObj  = new XmlToArray($xml_data); 
			//Creating Array 
			$arrayData = $xmlObj->createArray(); 
			
			
			if($arrayData['eparcel']['ratesAndServicesResponse'][0]['statusMessage'] == "OK") {
				
				$optionNb = count($arrayData['eparcel']['ratesAndServicesResponse'][0]['product']);
				
				for($i = 0; $i < $optionNb; $i++) {
					$name = $arrayData['eparcel']['ratesAndServicesResponse'][0]['product'][$i]['name'];
					$rate = $arrayData['eparcel']['ratesAndServicesResponse'][0]['product'][$i]['rate'];
					$returnArray[$name] = floatval($rate);
				}			
			}
			
			return $returnArray;		             
		
		}
	}	
}

function canadapost_add($wpsc_shipping_modules) {

	global $canadapost;
	$canadapost = new canadapost();

	$wpsc_shipping_modules[$canadapost->getInternalName()] = $canadapost;

	return $wpsc_shipping_modules;
}
	
add_filter('wpsc_shipping_modules', 'canadapost_add');
?>